package com.indicator.controller;

import com.baomidou.mybatisplus.extension.conditions.query.LambdaQueryChainWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.common.entity.Scene;
import com.common.entity.dic.IndicatorDic;
import com.common.entity.dic.AppDic;
import com.common.info.CodeMsg;
import com.indicator.entity.Indicator;
import com.indicator.entity.IndicatorLabel;
import com.indicator.entity.Label;
import com.indicator.info.Result;
import com.indicator.mapper.IndicatorLabelMapper;
import com.indicator.mapper.LabelMapper;
import com.indicator.repository.*;
import com.indicator.vo.IndicatorVo;
import com.indicator.info.Search;
import com.indicator.vo.DynamicTable;
import com.indicator.mapper.IndicatorMapper;
import com.indicator.service.DynamicTableService;
import com.indicator.service.SearchService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.*;

@Slf4j
@RestController
@RequestMapping("/indicator")
@Api(tags = "服务接口")
public class IndicatorController {
    @Resource
    private SceneRepository sceneRepository;
    @Resource
    private IndicatorDicRepository indicatorDicRepository;
    @Resource
    private AppDicRepository appDicRepository;
    @Resource
    private IndicatorRepository indicatorRepository;
    @Resource
    private IndicatorMapper indicatorMapper;
    @Resource
    private LabelRepository labelRepository;
    @Resource
    private LabelMapper labelMapper;
    @Resource
    private IndicatorLabelRepository indicatorLabelRepository;
    @Resource
    private IndicatorLabelMapper indicatorLabelMapper;
    @Resource
    private DynamicTableService dynamicTableService;
    @Resource
    private SearchService searchService;



    private void setSuccess(Result result){
        if(result.getCode() == null){
            result.setCode(CodeMsg.SUCCESS.getCode());
        }
        if(result.getMsg() == null){
            result.setMsg(CodeMsg.SUCCESS.getMessage());
        }
    }

    private void setListSuccess(Result result,List list){
        setSuccess(result);

        if(list != null){
            HashMap<String, Object>  hashMap =new HashMap<>();
            hashMap.put("data", list);
            hashMap.put("totalCount", result.getTotal());
            result.setData(hashMap);
            if(result.getTotal() == null){
                result.setTotal((long) list.size());
            }
        }else {
            result.setTotal(0L);
        }
    }

    private void setFailure(Result result){
        if(result.getCode() == null){
            result.setCode(CodeMsg.FAILURE.getCode());
        }
        if(result.getMsg() == null){
            result.setMsg(CodeMsg.FAILURE.getMessage());
        }
        result.setTotal(0L);
    }

    private void setExFailure(Result result,Exception e){
        result.setData(e.getMessage());
        setFailure(result);
    }


    /**
     * 关联场景查询
     * @return Result
     */
    @ApiOperation(value = "获取关联场景",notes = "获取关联场景列表信息")
    @GetMapping("get_scenes")
    public Result getScenes() {
        Result result = new Result();
        try {
            List<Scene> list = sceneRepository.list();
            setListSuccess(result,list);
            log.info("-------- 获取关联场景成功 ");
        }catch (Exception e){
            log.error("-------- 获取关联场景异常 : "+e.getMessage(),e);
            setExFailure(result,e);
        }
        return result;
    }

    /**
     * 指标类型查询
     * @return Result
     */
    @ApiOperation(value = "获取指标类型",notes = "获取指标类型列表信息")
    @GetMapping("get_indicator_dic")
    public Result getIndicatorDic() {
        Result result = new Result();
        try {
            List<IndicatorDic> list = indicatorDicRepository.list();
            setListSuccess(result,list);
            log.info("-------- 获取指标类型成功 ");
        }catch (Exception e){
            log.error("-------- 获取指标类型异常 : "+e.getMessage(),e);
            setExFailure(result,e);
        }
        return result;
    }

    /**
     * 关联应用模块类型查询
     * @return Result
     */
    @ApiOperation(value = "获取关联应用模块类型",notes = "获取关联应用模块类型列表信息")
    @GetMapping("get_app_p_dic")
    public Result getAppPDic() {
        Result result = new Result();
        try {
            List<AppDic> list = appDicRepository.lambdaQuery().isNull(AppDic::getParentId).list();
            setListSuccess(result,list);
            log.info("-------- 获取关联应用模块成功 ");
        }catch (Exception e){
            log.error("-------- 获取关联应用模块异常 : "+e.getMessage(),e);
            setExFailure(result,e);
        }
        return result;
    }

    /**
     * 关联功能类型查询
     * @return Result
     */
    @ApiOperation(value = "获取关联功能类型",notes = "获取关联功能类型列表信息")
    @ApiImplicitParam(name = "p_id",value = "关联应用模块类型 id，Content-Type:application/x-www-form-urlencoded",required = true)
    @GetMapping("get_app_c_dic")
    public Result getAppCDic(Integer p_id) {
        Result result = new Result();
        try {
            List<AppDic> list = appDicRepository.lambdaQuery().eq(AppDic::getParentId,p_id).list();
            setListSuccess(result,list);
            log.info("-------- 获取联功能成功 ");
        }catch (Exception e){
            log.error("-------- 获取联功能异常,p_id : "+p_id,e);
            setExFailure(result,e);
        }
        return result;
    }


    /**
     * 关联应用表信息查询
     * @return Result
     */
    @ApiOperation(value = "获取关联应用表信息",notes = "获取关联应用表信息")
    @GetMapping("get_app_tables")
    public Result getAppTables() {
        Result result = new Result();
        try {
            List<String> list = searchService.getTableNames();
            setListSuccess(result,list);
            log.info("-------- 获取关联应用表成功 ");
        }catch (Exception e){
            log.error("-------- 获取关联应用表异常 : "+e.getMessage(),e);
            setExFailure(result,e);
        }
        return result;
    }

    /**
     * 关联应用表信息查询
     * @return Result
     */
    @ApiOperation(value = "获取关联应用表字段信息",notes = "获取关联应用表字段信息")
    @ApiImplicitParam(name = "tableName",value = "关联应用表名称，Content-Type:application/x-www-form-urlencoded",required = true)
    @GetMapping("get_app_table_columns")
    public Result getAppTableColumns(String tableName) {
        Result result = new Result();
        try {
            List<String> list = searchService.getColumns(tableName);
            setListSuccess(result,list);
            log.info("-------- 获取关联应用表字段成功 ");
        }catch (Exception e){
            log.error("-------- 获取关联应用表字段异常 : "+e.getMessage(),e);
            setExFailure(result,e);
        }
        return result;
    }


    /**
     * 关联应用模块类型查询
     * @return Result
     */
    @ApiOperation(value = "获取一级标签",notes = "获取一级标签列表信息")
    @GetMapping("get_p_label")
    public Result getPLabel() {
        Result result = new Result();
        try {
            List<Label> list = labelRepository.lambdaQuery().isNull(Label::getParentId).list();
            setListSuccess(result,list);
            log.info("-------- 获取一级标签成功 ");
        }catch (Exception e){
            log.error("-------- 获取一级标签异常 : "+e.getMessage(),e);
            setExFailure(result,e);
        }
        return result;
    }


    /**
     * 标签查询
     * @return Result
     */
    @ApiOperation(value = "获取子级标签",notes = "获取子级标签列表信息")
    @ApiImplicitParam(name = "p_id",value = "标签 id，Content-Type:application/x-www-form-urlencoded",required = true)
    @GetMapping("get_c_label")
    public Result getCLabel(Integer p_id) {
        Result result = new Result();
        try {
            List<Label> list = labelRepository.lambdaQuery().eq(Label::getParentId,p_id).list();
            setListSuccess(result,list);
            log.info("-------- 获取子级标签成功 ");
        }catch (Exception e){
            log.error("-------- 获取子级标签异常,p_id : "+p_id,e);
            setExFailure(result,e);
        }
        return result;
    }

    private List<Indicator> getValidList(Indicator indicator){
        List<Indicator> list = indicatorRepository.lambdaQuery()
                .eq(Indicator::getSceneId,indicator.getSceneId())
                .eq(Indicator::getIndicatorDicId,indicator.getIndicatorDicId())
                .eq(Indicator::getAppDicPId,indicator.getAppDicPId())
                .eq(Indicator::getAppDicCId,indicator.getAppDicCId())
                .eq(Indicator::getDynamicColumn,indicator.getDynamicColumn())
                .list();
        return list;
    }

    private int validSave(Indicator indicator){
        List<Indicator> list = getValidList(indicator);
        if(list != null && list.size() > 0){
            return 1;
        }else {
            //true
            return 0;
        }
    }

    private int validUpdate(Indicator indicator,Indicator oldIndicator){
        List<Indicator> list5 = getValidList(indicator);
        List<Indicator> list6 = indicatorRepository.lambdaQuery()
                .eq(Indicator::getSceneId,indicator.getSceneId())
                .eq(Indicator::getIndicatorDicId,indicator.getIndicatorDicId())
                .eq(Indicator::getAppDicPId,indicator.getAppDicPId())
                .eq(Indicator::getAppDicCId,indicator.getAppDicCId())
                .eq(Indicator::getDynamicColumn,indicator.getDynamicColumn())
                .eq(Indicator::getId,indicator.getId())
                .list();
        List<Indicator> list4 = indicatorRepository.lambdaQuery()
                .eq(Indicator::getSceneId,indicator.getSceneId())
                .eq(Indicator::getIndicatorDicId,indicator.getIndicatorDicId())
                .eq(Indicator::getAppDicPId,indicator.getAppDicPId())
                .eq(Indicator::getAppDicCId,indicator.getAppDicCId())
                .list();

        if(oldIndicator != null){
            if(list5 != null && list5.size() > 0){
                if(list6 != null && list6.size() > 0){
                    //自身更新，true
                    return 1;
                }else {
                    //en name 重复
                    return 2;
                }
            }else {
                String tableName = dynamicTableService.getTableName(indicator);
                String oldTableName = dynamicTableService.getTableName(oldIndicator);
                if(list4 != null && list4.size() > 0 && oldTableName.equals(tableName)){
                    //代表同一张表
                    //rename true
                    return 4;
                }else {
                    //2张表联动更新 true
                    return 0;
                }
            }
        }else {
            //id不存在
            return 3;
        }
    }

    /**
     * 校验指标动态表名称
     * @return Result
     */
    @ApiOperation(value = "校验指标动态表名称",notes = "新建0和1代表合法；更新0代表合法")
    @PostMapping("valid_dynamic_column")
    public Result validDynamicColumn(@RequestBody Indicator indicator) {
        Result result = new Result();
        try {
            int flag;
            if(indicator.getId() != null){
                Indicator oldIndicator = indicatorRepository.lambdaQuery().eq(Indicator::getId,indicator.getId()).one();
                flag = validUpdate(indicator,oldIndicator);
                if(0 == flag || 1 == flag){
                    log.info("-------- 指标动态字段可用,name :"+indicator.getDynamicColumn());
                    result.setCode(CodeMsg.SUCCESS.getCode());
                    result.setMsg("指标英文名称可用");
                }else {
                    log.error("-------- 指标动态字段重名！！！ name :"+indicator.getDynamicColumn());
                    result.setCode(CodeMsg.FAILURE.getCode());
                    result.setMsg("指标动态字段重名");
                }
            }else {
                flag = validSave(indicator);
                if(0 == flag){
                    log.info("-------- 指标动态字段可用,name :"+indicator.getDynamicColumn());
                    result.setCode(CodeMsg.SUCCESS.getCode());
                    result.setMsg("指标英文名称可用");
                }else {
                    log.error("-------- 指标动态字段重名！！！ name :"+indicator.getDynamicColumn());
                    result.setCode(CodeMsg.FAILURE.getCode());
                    result.setMsg("指标动态字段重名");
                }
            }

            result.setTotal(0L);
            result.setData(flag);
        }catch (Exception e){
            log.error("-------- 获取联功能异常,indicator : "+indicator,e);
            setExFailure(result,e);
        }
        return result;
    }

    @ApiOperation(value = "获取标签")
    @ApiImplicitParams({@ApiImplicitParam(name = "table", value = "表名称", required = true),@ApiImplicitParam(name = "level", value = "级别", required = true)})
    @GetMapping("get_label")
    public Result getLabel(String table,int level) {
        Result result = new Result();
        try {
            List<Label> list = labelRepository.lambdaQuery().eq(Label::getTableName,table).eq(Label::getLabelLevel,level).list();
            setListSuccess(result,list);
        }catch (Exception e){
            log.error("-------- 获取联功能异常,table : "+table+" level:"+level,e);
            setExFailure(result,e);
        }
        return result;
    }


    private Result setErrorResult(Result result,String tableName,Indicator indicator){
        result.setCode(CodeMsg.FAILURE.getCode());
        result.setMsg("参数错误");
        result.setTotal(0L);
        result.setData("保存指标动态表字段重复 table name : " + tableName +" , column ："+indicator.getDynamicColumn());
        log.error("-------- 保存指标动态表字段重复 table name : " + tableName +" , column ："+indicator.getDynamicColumn());
        return result;
    }


    /**
     * 保存或修改指标
     * @return Result
     */
    @ApiOperation(value = "保存或修改指标",notes = "保存指标")
    @PostMapping("save_update_indicator")
    @Transactional
    public Result saveUpdateIndicator(@RequestBody IndicatorVo indicator) {
        Result result = new Result();
        Date date = new Date();
        boolean updateFlag = true;
        int updateNum = 6;
        Indicator oldIndicator = null;
        String tableName = null;
        try {
            tableName = dynamicTableService.getTableName(indicator);
            if("create_time".equals(indicator.getDynamicColumn()) || "update_time".equals(indicator.getDynamicColumn())){
                return setErrorResult(result,tableName,indicator);
            }

            //1、校验指标名称会不会重复
            if(indicator.getId() == null){
                updateFlag = false;

                if(validSave(indicator) == 0){
                    indicator.setCreateTime(date);
                    indicator.setUpdateTime(date);
                }else {
                    return setErrorResult(result,tableName,indicator);
                }
            }else {
                oldIndicator = indicatorRepository.lambdaQuery().eq(Indicator::getId,indicator.getId()).one();
                updateNum = validUpdate(indicator,oldIndicator);
                if(updateNum == 0 || updateNum == 1){
                    indicator.setUpdateTime(date);
                }else {
                    if(updateNum == 2){
                        return setErrorResult(result,tableName,indicator);
                    }else if(updateNum == 3){
                        result.setData("在数据库中找不到需要更新的 id = " + indicator.getId());
                        log.error("-------- 在数据库中找不到需要更新的 id = " + indicator.getId());
                        result.setCode(CodeMsg.FAILURE.getCode());
                        result.setMsg("参数错误");
                        result.setTotal(0L);
                        return result;
                    }
                }
            }
            log.info("--------1、 指标动态字段 : " + tableName +"_"+indicator.getDynamicColumn()+" 校验成功");

            //2、创建表 , 后面的异常要向上抛出
            indicator.setDynamicTable(tableName);
            List<Label> label_list = indicator.getLableList();
            if(label_list != null && label_list.size()>0) {
                Collections.sort(label_list, (Label1, Label2) -> Label1.getLabelLevel() - Label2.getLabelLevel());
            }

            boolean indicator_flag = indicatorRepository.saveOrUpdate(indicator);
            if(indicator_flag){
                if(!updateFlag){
                    if(label_list == null || label_list.size()==0){
                        log.error("-------- 保存指标异常， 保存指标时， 标签不能为空，最少包含一级标签");
                        throw new RuntimeException("保存指标时， 标签不能为空，最少包含一级标签");
                    }
                }

                if(label_list != null && label_list.size()>0){
                    boolean saveTmpFlag = false;
                    for (int i = 0; i < label_list.size(); i++) {
                        if(!updateFlag){
                            if(i > 0 && i <(label_list.size())) {
                                label_list.get(i).setParentId(label_list.get(i-1).getId());
                            }
                            label_list.get(i).setCreateTime(date);
                            saveTmpFlag = true;
                        }else {
                            if (label_list.get(i).getId() == null) {
                                // 新增指标
                                if(i > 0 && i <(label_list.size())) {
                                    label_list.get(i).setParentId(label_list.get(i-1).getId());
                                }
                                label_list.get(i).setCreateTime(date);
                                saveTmpFlag = true;
                            } else {
                                List<IndicatorLabel> I_L_list = indicatorLabelRepository.lambdaQuery().eq(IndicatorLabel::getIndicatorId, indicator.getId()).list();
                                boolean ILFlag = false;
                                for (int j = 0; j < I_L_list.size(); j++) {
                                    if (label_list.get(i).getId().equals(I_L_list.get(j).getLabelId())) {
                                        ILFlag = true;
                                    }
                                }
                                if (!ILFlag) {
                                    log.error("-------- 保存指标异常， 更新指标id : " + label_list.get(i).getId() + "无效，请输入正确的指标id");
                                    throw new RuntimeException("更新指标id : " + label_list.get(i).getId() + "无效，请输入正确的指标id");
                                }
                                Label label = labelRepository.getById(label_list.get(i).getId());
                                label_list.get(i).setParentId(label.getParentId());
                            }
                        }
                        label_list.get(i).setTableName(tableName);
                        label_list.get(i).setUpdateTime(date);
                        if(i == label_list.size()-1){
                            label_list.get(i).setColumnName(indicator.getDynamicColumn());
                        }
                        boolean label_flag = labelRepository.saveOrUpdate(label_list.get(i));

                        System.out.println("___"+label_list.get(i).getId());
                        if(label_flag){
                            if(!updateFlag || saveTmpFlag){
                                IndicatorLabel indicator_label = new IndicatorLabel();
                                indicator_label.setIndicatorId(indicator.getId());
                                indicator_label.setLabelId(label_list.get(i).getId());
                                indicator_label.setCreateTime(date);
                                indicator_label.setUpdateTime(date);
                                indicatorLabelRepository.saveOrUpdate(indicator_label);
                            }
                        }

                    }
                }

            }else {
                log.error("-------- 保存指标异常， 更新指标失败， table : " + tableName +" , column : "+indicator.getDynamicColumn());
                throw new RuntimeException("更新指标失败， table : " + tableName +" , column : "+indicator.getDynamicColumn());
            }
            log.info("--------2、 动态标签创建成功");

            if(StringUtils.hasLength(indicator.getDynamicColumn())){
                if(2 == indicator.getAppDicPId()){
                    result.setData("创建动态表， table : " + tableName +" , column : "+indicator.getDynamicColumn());
                    if(updateFlag){
                        //updateNum == 1 自身更新，无需修改表
                        if(updateNum == 0){
                            dynamicTableService.executeUpdate0(indicator,oldIndicator);
                        }else if(updateNum == 4){
                            dynamicTableService.executeUpdate1(indicator,oldIndicator);
                        }
                        result.setMsg("更新成功");
                        log.info("-------- 更新成功， table : " + tableName +" , column : "+indicator.getDynamicColumn());
                    }else {
                        dynamicTableService.executeSave(indicator);
                        result.setMsg("保存成功");
                        log.info("-------- 保存指标成功， table : " + tableName +" , column : "+indicator.getDynamicColumn());
                    }
                }else {
                    log.info("-------- 指标类型 :" + indicator.getAppDicPId() +" 动态表未改动");
                }
            }else {
                log.info("-------- 英文字段为空，动态表未改动");
            }

            result.setCode(CodeMsg.SUCCESS.getCode());
            result.setTotal(0L);
        }catch (Exception e){
            if(updateFlag){
                log.error("-------- 更新指标异常， table : " + tableName +" , column : "+indicator.getDynamicColumn());
                throw new RuntimeException("更新指标异常"+e.getMessage());
            }else {
                log.error("-------- 保存指标异常， table : " + tableName +" , column : "+indicator.getDynamicColumn());
                throw new RuntimeException("保存指标异常"+e.getMessage());
            }
        }
        return result;
    }



    /**
     * 删除指标
     * @return Result
     */
    @ApiOperation(value = "删除指标",notes = "删除指标")
    @ApiImplicitParam(name = "id",value = "关联应用表名称，Content-Type:application/x-www-form-urlencoded",required = true)
    @GetMapping("delete_indicator")
    @Transactional
    public Result deleteIndicator(Integer id) {
        Result result = new Result();
        try {
            Indicator indicator = indicatorRepository.lambdaQuery().eq(Indicator::getId,id).one();
            if(indicator == null){
                result.setMsg("删除失败");
                result.setData("无效的指标ID : "+id);
                setFailure(result);
                return result;
            }
            boolean delete_indicator_flag = indicatorRepository.removeById(indicator);
            if(delete_indicator_flag){
                //1、根据依赖关系删除标签
                LambdaQueryChainWrapper<IndicatorLabel> wrapper = indicatorLabelRepository.lambdaQuery().eq(IndicatorLabel::getIndicatorId,id);
                List<IndicatorLabel> lable_id_list = wrapper.select(IndicatorLabel::getLabelId).list();
                for (IndicatorLabel indicator_label : lable_id_list) {
                    if(indicatorLabelRepository.lambdaQuery().eq(IndicatorLabel::getLabelId,indicator_label.getLabelId()).list().size() == 1){
                        boolean label_flag = labelRepository.removeById(indicator_label.getLabelId());
                        if(!label_flag){
                            log.error("-------- 删除标签失败，标签 id : "+indicator_label.getLabelId());
                            throw new RuntimeException("删除标签失败，标签 id : "+indicator_label.getLabelId());
                        }
                    }
                }
                //2、批量删除中间关系
                boolean i_l_flag = indicatorLabelRepository.removeBatchByIds(wrapper.select(IndicatorLabel::getId).list());

                if(i_l_flag){
                    //3、删除动态字段
                    if(2 == indicator.getAppDicPId()){
                        dynamicTableService.deleteTable(indicator);
                        result.setCode(CodeMsg.SUCCESS.getCode());
                        result.setMsg("删除成功");
                        result.setData("删除指标成功");
                        result.setTotal(0L);
                        log.info("-------- 删除指标成功 ");
                    }
                }else {
                    log.error("-------- 删除多对多映射关系失败");
                    throw new RuntimeException("删除多对多映射关系失败");
                }

            }else {
                log.error("-------- 删除指标失败,indicator : "+indicator);
                throw new RuntimeException("删除指标失败");
            }
        }catch (Exception e){
            log.error("-------- 删除指标失败,indicator id : "+id,e);
            throw new RuntimeException("-------- 删除指标失败,indicator id : "+id);
        }
        return result;
    }



    /**
     * 查询指标列表
     * @return Result
     */
    @ApiOperation(value = "查询指标列表")
    @PostMapping("search_indicator")
    public Result searchIndicator(@RequestBody Search search) {
        Result result = new Result();
        try {
            Page<IndicatorVo> query = new Page(search.getCurrentPage()-1, search.getPageSize());
            Page<IndicatorVo> page = indicatorMapper.getIndicatorPage(query,search.getValue());
            for (IndicatorVo vo : page.getRecords()) {
                LambdaQueryChainWrapper<IndicatorLabel> wrapper = indicatorLabelRepository.lambdaQuery().eq(IndicatorLabel::getIndicatorId,vo.getId());
                List<Integer> list = new ArrayList();
                for (IndicatorLabel indicatorLabel : wrapper.select(IndicatorLabel::getLabelId).list()) {
                    list.add(indicatorLabel.getLabelId());
                }
                List<Label> labelList = labelRepository.lambdaQuery().in(Label::getId,list).orderByAsc(Label::getLabelLevel).list();
                vo.setLableList(labelList);
            }
            result.setTotal(page.getTotal());
            setListSuccess(result,page.getRecords());
            log.info("-------- 获取关联应用表字段成功 ");
        }catch (Exception e){
            log.error("-------- 获取联功能异常,search : "+search,e);
            setExFailure(result,e);
        }
        return result;
    }


    /**
     * 查询指标列表
     * @return Result
     */
    @ApiOperation(value = "查询指标动态表")
    @GetMapping("search_dynamic_tables")
    public Result getDynamicTables() {
        Result result = new Result();
        try {
            List<DynamicTable> list = searchService.getTableInfo();
            List<Scene> scene_list = sceneRepository.list();
            List<IndicatorDic> indicator_list = indicatorDicRepository.list();
            List<AppDic> app_p_list = appDicRepository.lambdaQuery().isNull(AppDic::getParentId).list();

            for (DynamicTable dynamicTable : list) {
                String scene_id = dynamicTable.getTableNameEn().split("_")[1];
                String indicator_id = dynamicTable.getTableNameEn().split("_")[2];
                String app_p_id = dynamicTable.getTableNameEn().split("_")[3];
                String app_c_id = dynamicTable.getTableNameEn().split("_")[4];
                StringBuffer tableName = new StringBuffer();
                for (Scene scene : scene_list) {
                    if(scene_id.equals(scene.getId()+"")){
                        dynamicTable.setSceneName(scene.getSceneName());
                        tableName.append(scene.getSceneName()+"_");
                        break;
                    }
                }
                for (IndicatorDic indicatorDic : indicator_list) {
                    if(indicator_id.equals(indicatorDic.getId()+"")){
                        tableName.append(indicatorDic.getIndicatorType()+"_");
                        break;
                    }
                }
                for (AppDic appDic : app_p_list) {
                    if(app_p_id.equals(appDic.getId()+"")){
                        tableName.append(appDic.getAppType()+"_");
                        List<AppDic> app_c_list = appDicRepository.lambdaQuery().eq(AppDic::getParentId,appDic.getId()).list();
                        for (AppDic dic : app_c_list) {
                            if(app_c_id.equals(dic.getId()+"")){
                                tableName.append(dic.getAppType());
                                break;
                            }
                        }
                        break;
                    }
                }

                dynamicTable.setTableNameCn(tableName.toString());
            }

            result.setTotal((long) list.size());
            setListSuccess(result,list);
            log.info("-------- 查询指标动态表成功 ");
        }catch (Exception e){
            log.error("-------- 查询指标动态表异常",e);
            setExFailure(result,e);
        }
        return result;
    }


}
