package com.indicator.filter;

import com.alibaba.fastjson2.JSONObject;
import com.common.entity.User;
import com.common.info.CodeMsg;
import com.indicator.info.Result;
import com.indicator.repository.UserRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.PrintWriter;

@Slf4j
public class TokenInterceptor implements HandlerInterceptor {

    private UserRepository userRepository;
    private String[] whiteList = {"/openapi/"};

    public TokenInterceptor(UserRepository userRepository){
        this.userRepository = userRepository;
    }



    private void errorFilter(HttpServletResponse response){
        response.setCharacterEncoding("UTF-8");
        response.setContentType("application/json; charset=utf-8");
        response.setHeader("Access-Control-Allow-Origin", "*");
        response.setHeader("Access-Control-Allow-Headers", "*");

        Result result = new Result();
        result.setCode(CodeMsg.AUTH_FAILURE.getCode());
        result.setMsg(CodeMsg.AUTH_FAILURE.getMessage());
        PrintWriter out = null;
        try {
            out = response.getWriter();
            out.write(JSONObject.toJSONString(result));
        } catch (Exception e) {
            e.printStackTrace();
        }finally {
            if(out != null){
                out.flush();
                out.close();
            }
        }
    }

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) {
        // 如果不是映射到方法直接通过
        if (!(handler instanceof HandlerMethod)) {
            return true;
        }

        for (String url : whiteList) {
            if(request.getRequestURL().toString().contains(url)){
                return true;
            }
        }


        //他们校验的居然不是token串，而是去后台校验一个用户id ？？？
        String tmpId = request.getHeader("userId");
        if(tmpId!=null){
            Integer userId = Integer.valueOf(tmpId);
            log.info("获取到用户信息 userId : " + userId);
            if(userId!=null){
                User user = userRepository.lambdaQuery().eq(User::getId,userId).one();
                if(user!=null){
                    return true;
                }else{
                    errorFilter(response);
                    return false;
                }
            }else {
                errorFilter(response);
                return false;
            }
        }else {
            errorFilter(response);
            return false;
        }
    }

}
