package com.indicator.service;

import com.indicator.entity.Indicator;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;

@Service
@Slf4j
public class DynamicTableService {
    @Resource
    private DataSource dataSource;

    public static String getTableName(Indicator indicator){
        String tablename = "indicator_"+indicator.getSceneId()
                +"_"+indicator.getIndicatorDicId()
                +"_"+indicator.getAppDicPId()
                +"_"+indicator.getAppDicCId()
                ;
        return tablename;
    }


    public void executeSave(Indicator indicator) {
        Connection conn = null;
        Statement stm = null;
        try {
            String tableName = getTableName(indicator);
            conn = dataSource.getConnection();
            stm = conn.createStatement();
            log.info("---------- 创建表 table name ："+tableName);
            String create_sql = "create table if not exists "+tableName+" (id int4 generated always as identity primary key, bm varchar(255) null, " +
                    "create_time timestamp, update_time timestamp,"+indicator.getDynamicColumn()+" varchar(255))";
            log.info("---------- 创建表 sql："+create_sql);
            String add_sql = "alter table "+tableName+" add column if not exists "+indicator.getDynamicColumn()+" varchar(255)";
            log.info("---------- 添加字段 sql："+add_sql);
//            String comment_id_sql = "comment on column "+tableName+".id is 'ID序号'";
//            String comment_column_sql = "comment on column "+tableName+"."+indicator.getDynamicColumn()+" is '"+indicator.getDynamicColumnCh()+"'";
//            log.info("---------- 添加ID备注 sql："+comment_id_sql);
//            log.info("---------- 添加column备注 sql："+comment_column_sql);

            stm.executeUpdate(create_sql);
            stm.executeUpdate(add_sql);
//            stm.executeUpdate(comment_id_sql);
//            stm.executeUpdate(comment_column_sql);
        }catch (Exception e){
            throw new RuntimeException("执行sql失败",e);
        }finally {
            try {
                if(stm != null) stm.close();
            }catch (Exception e){
                throw new RuntimeException("stm.close() 异常！");
            }
            try {
                if(conn != null) conn.close();
            }catch (Exception e){
                throw new RuntimeException("conn.close() 异常！");
            }
        }
    }

    public void executeUpdate0(Indicator indicator,Indicator oldIndicator) throws Exception{
        deleteTable(oldIndicator);
        executeSave(indicator);
    }

    public void executeUpdate1(Indicator indicator,Indicator oldIndicator) {
        Connection conn = null;
        Statement stm = null;
        try {
            String tableName = getTableName(indicator);
            conn = dataSource.getConnection();
            stm = conn.createStatement();
            log.info("---------- 更新字段表名称 table name ："+tableName);
            String alter_sql = "alter table "+tableName+" rename "+oldIndicator.getDynamicColumn()+" to "+indicator.getDynamicColumn();
            log.info("---------- 更新字段名称 sql："+alter_sql);
            stm.executeUpdate(alter_sql);
        }catch (Exception e){
            throw new RuntimeException("执行sql失败",e);
        }finally {
            try {
                if(stm != null) stm.close();
            }catch (Exception e){
                throw new RuntimeException("stm.close() 异常！");
            }
            try {
                if(conn != null) conn.close();
            }catch (Exception e){
                throw new RuntimeException("conn.close() 异常！");
            }
        }
    }

    public void deleteTable(Indicator indicator) {
        Connection conn = null;
        Statement stm = null;
        String tableName = null;
        try {
            Set<String> set = new LinkedHashSet();
            List<String> columnList = new ArrayList();
            tableName = getTableName(indicator);
            conn = dataSource.getConnection();
            stm = conn.createStatement();

            DatabaseMetaData metaData = conn.getMetaData();
            ResultSet resultSet = metaData.getColumns(null,null,tableName,null);
            while (resultSet.next()) {
                set.add(resultSet.getString("COLUMN_NAME"));
            }
            for (String column : set) {
                columnList.add(column);
            }

            String alter_sql = "alter table "+tableName+" drop column if exists "+indicator.getDynamicColumn();
            String drop_sql = "drop table if exists "+tableName;

            if(columnList.size()>5){
                log.info("---------- 删除表字段 sql："+alter_sql);
                stm.executeUpdate(alter_sql);
            }else {
                log.info("---------- 删除表 sql："+drop_sql);
                stm.executeUpdate(drop_sql);
            }

        }catch (Exception e){
            throw new RuntimeException("删除动态表失败，table name : "+tableName,e);
        }finally {
            try {
                if(stm != null) stm.close();
            }catch (Exception e){
                throw new RuntimeException("stm.close() 异常！");
            }
            try {
                if(conn != null) conn.close();
            }catch (Exception e){
                throw new RuntimeException("conn.close() 异常！");
            }
        }
    }


}
