package com.indicator.service;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONException;
import com.alibaba.fastjson2.JSONObject;
import com.common.info.Parameter;
import com.indicator.entity.Label;
import com.indicator.repository.LabelRepository;
import com.indicator.tool.TreeTool;
import com.indicator.vo.DynamicTable;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.*;

@Service
@Slf4j
public class SearchService {
    @Resource
    private DataSource dataSource;
    @Resource
    private LabelRepository labelRepository;


    public List<String> getTableNames() throws Exception{
        Connection conn = null;
        ResultSet result = null;
        try {
            conn = dataSource.getConnection();
            DatabaseMetaData metaData = conn.getMetaData();
            Set<String> set = new LinkedHashSet();
            result = metaData.getTables(null, null, null, new String[]{"TABLE"});
            while (result.next()) {
                set.add(result.getString("table_name"));
            }
            List<String> list = new ArrayList();
            for (String column : set) {
                list.add(column);
            }
            return list;
        }catch (Exception e){
            throw new Exception("获取数据库列表失败！",e);
        }finally {
            try {
                if(result != null) result.close();
            }catch (Exception e){
                throw new Exception("获取数据库列表失败，result.close() 异常！");
            }
            try {
                if(conn != null) conn.close();
            }catch (Exception e){
                throw new Exception("获取数据库列表失败，conn.close() 异常！");
            }
        }
    }

    public List<String> getColumns(String tableName) throws Exception{
        Connection conn = null;
        ResultSet result = null;
        try {
            Set<String> set = new LinkedHashSet();
            conn = dataSource.getConnection();
            DatabaseMetaData metaData = conn.getMetaData();
            result = metaData.getColumns(null,null,tableName,null);
            while (result.next()) {
                //COLUMN_NAME：column name and size  TYPE_NAME：data type name
                set.add(result.getString("column_name"));
            }
            List<String> list = new ArrayList();
            for (String column : set) {
                list.add(column);
            }
            return list;
        }catch (Exception e){
            throw new Exception("获取数据库列表失败！",e);
        }finally {
            try {
                if(result != null) result.close();
            }catch (Exception e){
                throw new Exception("获取数据库列表失败，result.close() 异常！");
            }
            try {
                if(conn != null) conn.close();
            }catch (Exception e){
                throw new Exception("获取数据库列表失败，conn.close() 异常！");
            }
        }
    }

    public List<DynamicTable> getTableInfo() throws Exception{
        List<String> allTableNames = getTableNames();
        List<String> tableNames = new ArrayList();
        for (String tableName : allTableNames) {
            if(tableName.startsWith("indicator_") && !tableName.endsWith("_dic")){
                tableNames.add(tableName);
            }
        }


        List<DynamicTable> list = new ArrayList();
        Connection conn = null;
        try {
            conn = dataSource.getConnection();

            for (String tableName : tableNames) {
                DynamicTable dynamicTable = new DynamicTable();

                JSONArray columnArray = TreeTool.listToTree(JSONArray
                        .parseArray(JSON.toJSONString(labelRepository.lambdaQuery()
                                .eq(Label::getTableName,tableName).list(), Parameter.FORMAT)),"id","parentId","children");
                List<JSONObject> lableList = TreeTool.processArray(columnArray,new ArrayList());


                StringBuffer columns = new StringBuffer();
                List<String> columnList = new ArrayList();
                for (JSONObject jsonObject : lableList) {
                    columnList.add(jsonObject.get("columnName").toString());
                }
                for (int i = 0; i < columnList.size(); i++) {
                    columns.append(columnList.get(i)+",");
                }
                columnList.add("bm");
                columnList.add("create_time");
                columnList.add("update_time");
                String sql = "select "+columns+"bm, create_time,update_time from "+tableName;
                log.info("---- sql : "+sql);

                List<List<String>> lists = new ArrayList();
                PreparedStatement stm = conn.prepareStatement(sql, ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
                stm.setFetchSize(10000);
                stm.setFetchDirection(ResultSet.FETCH_FORWARD);
                ResultSet rs  = stm.executeQuery();
                while (rs.next()) {
                    List<String> values = new ArrayList();
                    for (int i = 1; i <= rs.getMetaData().getColumnCount(); i++) {
                        Object obj = rs.getObject(i);
                        if(obj == null){
                            values.add("");
                        }else{
                            if(i >= rs.getMetaData().getColumnCount()-1){
                                values.add(Parameter.DATE_FORMAT.format(obj));
                            }else {
                                String tmpData = (obj+"").trim().replace("\n","\\n").replace("\t","\\t");
                                if("null".equals(tmpData)) {
                                    values.add("");
                                }else{
                                    values.add(tmpData);
                                }
                            }
                        }
                    }
                    lists.add(values);
                }
                rs.close();
                stm.close();

                dynamicTable.setTableNameEn(tableName);
                dynamicTable.setLabelArray(columnArray);
                dynamicTable.setColumnList(columnList);
                dynamicTable.setValueList(lists);
                list.add(dynamicTable);
            }
            return list;

        }catch (Exception e){
            throw new Exception("获取动态表信息失败！",e);
        }finally {
            try {
                if(conn != null) conn.close();
            }catch (Exception e){
                throw new Exception("获取动态表信息失败，conn.close() 异常！");
            }
        }
    }


}
